' ########################################################################################
' Microsoft Windows
' File: AfxWin.inc
' Contents: Windows wrapper functions
' Compiler: Oxygen Basic 32 & 64 bit
' Copyright (c) 2018 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################

use corewin
use Afx/AfxWinData

' ========================================================================================
' Returns the Windows version
' Platform 1:
'   400 Windows 95
'   410 Windows 98
'   490 Windows ME
' Platform 2:
'   400 Windows NT
'   500 Windows 2000
'   501 Windows XP
'   502 Windows Server 2003
'   600 Windows Vista and Windows Server 2008
'   601 Windows 7
'   602 Windows 8
'   603 Windows 8.1
'  1000 Windows 10
' Note: As Windows 95 and Windows NT return the same version number, we also need to call
' GetWindowsPlatform to differentiate them.
' ========================================================================================
' ========================================================================================
' GetVersion returns version 6.02 (Windows 8) in Windows 10 if the application has not a
' manifest.
' ========================================================================================
function AfxWindowsVersion () as long
   OSVERSIONINFOEXW osverinfo
   if RtlGetVersion(@osverinfo) = 0 then function = (osverinfo.dwMajorVersion + osverinfo.dwMinorVersion / 100) * 100
end function
' ========================================================================================

' ========================================================================================
' Returns the Windows platform
'   1 Windows 95/98/ME
'   2 Windows NT/2000/XP/Server/Vista/Windows 7
' ========================================================================================
function AfxWindowsPlatform () AS LONG
   dword dwVersion = GetVersion
   if dwVersion < &h80000000 then function = 2 else function = 1
end function
' ========================================================================================

' ========================================================================================
' Returns the Windows build
' ========================================================================================
function AfxWindowsBuild () as long
   OSVERSIONINFOEXW osverinfo
   if RtlGetVersion(@osverinfo) = 0 then function = osverinfo.dwBuildNumber
end function
' ========================================================================================

' ========================================================================================
' Returns TRUE if the Windows Platform is NT; FALSE, otherwise.
' ========================================================================================
function AfxIsPlatformNT () as bool
   OSVERSIONINFOW osvi
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOW)
   GetVersionExW(@osvi)
   if osvi.dwPlatformId = VER_PLATFORM_WIN32_NT then function = true
end function
' ========================================================================================

' ========================================================================================
' Returns the bitness of the operating system.
' On a 32-bit operating system, the value is 32 and on a 64-bit operating system it is 64.
' ========================================================================================
function AfxWindowsBitness () as long
   SYSTEM_INFO SysInfo
   GetNativeSystemInfo(@SysInfo)
   if SysInfo.wProcessorArchitecture = PROCESSOR_ARCHITECTURE_AMD64 then function = 64 else function = 32
end function
' ========================================================================================

' ========================================================================================
' Returns the version of specified file multiplied by 100, e.g. 601 for version 6.01.
' Example: DIM ver AS LONG = AfxGetFileVersion("COMCTL32.DLL")
' ========================================================================================
function AfxGetFileVersion (BYREF sFileName as string2) as long
   dim pvsffi AS VS_FIXEDFILEINFO PTR, dwHandle AS DWORD
   DWORD cbLen = GetFileVersionInfoSizeW(strptr(sFileName), @dwHandle)
   if cbLen = 0 then exit function
   SYS pVerInfo = HeapAlloc(GetProcessHeap, HEAP_ZERO_MEMORY, cbLen)
   if pVerInfo = 0 then exit function
   if GetFileVersionInfoW(strptr(sFileName), dwHandle, cbLen, pVerInfo) then
      if VerQueryValueW(pVerInfo, wstring("\"), @@pvsffi, @cbLen) then
         WORD wMajor = hiword(pvsffi.dwFileVersionMS)
         WORD wMinor = loword(pvsffi.dwFileVersionMS)
         function = (wMajor + wMinor / 100) * 100
      end if
   end if
   HeapFree(GetProcessHeap, 0, pVerInfo)
end function
' ========================================================================================

' ========================================================================================
' Returns the version of CommCtl32.dll multiplied by 100, e.g. 582 for version 5.82.
' ========================================================================================
function AfxComCtlVersion () as long
   function = AfxGetFileVersion("comctl32.dll")
end function
' ========================================================================================

' ========================================================================================
' Retrieves the path of the Windows directory.
' ========================================================================================
function AfxGetWinDir () as string2
   wchar wchWinDir[260]
   GetWindowsDirectoryW wchWinDir, countof(wchWinDir)
   return wchWinDir
end function
' ========================================================================================

' ========================================================================================
' Retrieves the name of the user associated with the current thread.
' ========================================================================================
function AfxGetUserName () as string2
   wchar buffer[260], dword dwBufLen = countof(buffer)
   GetUserNameW(buffer, @dwBufLen)
   return buffer
END FUNCTION
' ========================================================================================

#define MAX_COMPUTERNAME_LENGTH 15

' ========================================================================================
' Retrieves the computer name of the current system.
' ========================================================================================
function AfxGetComputerName () as string2
   wchar buffer[MAX_COMPUTERNAME_LENGTH + 1]
   dword dwBufLen = countof(buffer)
   GetComputerNameW(buffer, @dwBufLen)
   return buffer
end function
' ========================================================================================

#define ERROR_INSUFFICIENT_BUFFER 122

' ========================================================================================
' Returns the path of the program which is currently executing.
' The path name has not a trailing backslash, except if it is a drive, e.g. "C:\".
' ========================================================================================
function AfxGetExePath () as string2
   wchar buffer[260]
   GetModuleFileNameW(null, buffer, countof(buffer))
   if GetLastError = ERROR_INSUFFICIENT_BUFFER then exit function
   dim i as long, pch as word at strptr(buffer)
   for i = len(buffer) to 1 step -1
      if pch[i] = asc(":") or pch[i] = asc("\") or pch[i] = asc("/") then
         buffer = left(buffer, i)
         PathRemoveBackslashW(buffer)
         return buffer
      end if
   next
end function
' ========================================================================================

#define AfxExePath AfxGetExePath

' ========================================================================================
' Returns the path of the program which is currently executing.
' The path has a trailing backslash.
' ========================================================================================
function AfxGetExePathName () as string2
   wchar buffer[260]
   GetModuleFileNameW(null, buffer, countof(buffer))
   if GetLastError = ERROR_INSUFFICIENT_BUFFER then exit function
   dim i as long, pch as word at strptr(buffer)
   for i = len(buffer) to 1 step -1
      if pch[i] = asc(":") or pch[i] = asc("\") or pch[i] = asc("/") then
         return left(buffer, i)
      end if
   next
end function
' ========================================================================================

' ========================================================================================
' Returns the file name of the program which is currently executing.
' ========================================================================================
function AfxGetExeFileName () as string2
   wchar buffer[260]
   GetModuleFileNameW(null, buffer, countof(buffer))
   if GetLastError = ERROR_INSUFFICIENT_BUFFER then exit function
   dim i as long, pch as word at strptr(buffer)
   for i = len(buffer) to 1 step -1
      if pch[i] = asc(":") or pch[i] = asc("\") or pch[i] = asc("/") then
         dim n as long = len(buffer) - i + 1
         buffer = mid(buffer, i + 1, n)
         exit for
      end if
   next
   for i = len(buffer) to 1 step -1
      if pch[i] = asc(".") then
         buffer = left(buffer, i - 1)
         exit for
      end if
   next
   return buffer
end function
' ========================================================================================

' ========================================================================================
' Returns the file name and extension of the program which is currently executing.
' ========================================================================================
function AfxGetExeFileNameX () as string2
   wchar buffer[260]
   GetModuleFileNameW(null, buffer, countof(buffer))
   if GetLastError = ERROR_INSUFFICIENT_BUFFER then exit function
   dim i as long, pch as word at strptr(buffer)
   for i = len(buffer) to 1 step -1
      if pch[i] = asc(":") or pch[i] = asc("\") or pch[i] = asc("/") then
         dim n as long = len(buffer) - i + 1
         buffer = mid(buffer, i + 1, n)
         exit for
      end if
   next
   return buffer
end function
' ========================================================================================

' ========================================================================================
' Returns the extension (with a leading period) of the program which is currently executing.
' ========================================================================================
function AfxGetExeFileNameX () as string2
   wchar buffer[260]
   GetModuleFileNameW(null, buffer, countof(buffer))
   if GetLastError = ERROR_INSUFFICIENT_BUFFER then exit function
   dim i as long, pch as word at strptr(buffer)
   for i = len(buffer) to 1 step -1
      if pch[i] = asc(".") then
         dim n as long = len(buffer) - i + 1
         buffer = mid(buffer, i, n)
         exit for
      end if
   next
   return buffer
end function
' ========================================================================================

' ========================================================================================
' Returns the full path of the program which is currently executing.
' ========================================================================================
function AfxGetExeFullPath () as string2
   wchar buffer[260]
   GetModuleFileNameW(null, buffer, countof(buffer))
   if GetLastError = ERROR_INSUFFICIENT_BUFFER then exit function
   return buffer
end function
' ========================================================================================
